// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


/*
   Envia una peticion de interseccion sobre HTTP/XML para ihttp_server

   El cliente de esta aplicacion es: ihttp_server.p => Transporte: http::Transport.
*/
#include <iostream>

#include <string.h>

#include <anna/core/core.hpp>
#include <anna/comm/comm.hpp>

#include <anna/app/functions.hpp>

#include <anna/http/Request.hpp>
#include <anna/http/Response.hpp>
#include <anna/http/Handler.hpp>
#include <anna/http/Transport.hpp>
#include <anna/http/functions.hpp>

#include <anna/http/wims20/ClientSide.hpp>

class MyHandler : public http::Handler {
public:
   MyHandler () : http::Handler ("ihttp_client::MyHandler") {;}

private:
   http::Response a_httpResponse;

   void evRequest (ClientSocket&, const http::Request&) throw (RuntimeException) {;}
   void evResponse (ClientSocket&, const http::Response&) throw (RuntimeException);
};

class MyCommunicator : public Communicator {
public:
   MyCommunicator () : Communicator (), a_httpRequest ()
   {
      a_httpRequest.setMethod (http::Method::Type::Get);
      a_httpRequest.setURI ("ihttp_client");
   }

private:
   MyHandler a_httpHandler;
   http::Request a_httpRequest;

   void eventReceiveMessage (ClientSocket &, const Message&) throw (RuntimeException);
   void eventStartup () throw (RuntimeException);
};

class IHTTPClient : public anna::comm::Application {
public:
   IHTTPClient ();

   Server* getServer () const throw () { return a_server; }

private:
   MyCommunicator a_communicator;
   Server* a_server;

   void initialize () throw (RuntimeException);
   void run () throw (RuntimeException);
};

using namespace std;

int main (int argc, const char** argv)
{
   CommandLine& commandLine (CommandLine::instantiate ());
   IHTTPClient app;

   http::functions::initialize ();

   try {
      commandLine.initialize (argv, argc);
      commandLine.verify ();

      Logger::setLevel (Logger::Debug);
      Logger::initialize ("ihttp_client", new TraceWriter ("file.trace", 4096000));

      app.start ();
   }
   catch (Exception& ex) {
      cout << ex.asString () << endl;
   }

   return 0;
}

IHTTPClient::IHTTPClient () :
   Application ("wims20_client", "IHTTPClient", "2.0.0")
{
   CommandLine& commandLine (CommandLine::instantiate ());

   commandLine.add ("p", CommandLine::Argument::Mandatory, "Port to attend the answers.");
   commandLine.add ("a", CommandLine::Argument::Mandatory, "Direccion IP Puerto en el que el servidor atiende respuestas.");
   commandLine.add ("domain", CommandLine::Argument::Mandatory, "Domain indicado en la peticion WIMS 2.0");
   commandLine.add ("op", CommandLine::Argument::Mandatory, "operacion a realizar (sum,sub,mul,div)");
   commandLine.add ("x", CommandLine::Argument::Mandatory, "Primer operador");
   commandLine.add ("y", CommandLine::Argument::Mandatory, "Segundo operador");
   commandLine.add ("op", CommandLine::Argument::Mandatory, "operacion a realizar (sum,sub,mul,div)");
   commandLine.add ("trace", CommandLine::Argument::Optional, "Nivel de trazas (debug,warning, error,...)");
   commandLine.add ("path", CommandLine::Argument::Optional, "Path indicado en la peticion WIMS 2.0");
   commandLine.add ("m", CommandLine::Argument::Optional, "Metodo HTTP a usar");
}

void IHTTPClient::initialize () 
   throw (RuntimeException)
{
   CommandLine& cl (CommandLine::instantiate ());

   Network& network = Network::instantiate ();

   if (cl.exists ("trace"))
      Logger::setLevel (Logger::asLevel (cl.getValue ("trace")));

   Host* host = network.find_host("host000");
   host->assign (network.find (Device::asAddress (cl.getValue ("a"))));
   a_server = host->createServer ("http_server", cl.getIntegerValue ("p"), true, &http::Transport::getFactory ());
}

void IHTTPClient::run ()
   throw (RuntimeException)
{
   a_communicator.accept ();
}

void MyCommunicator::eventStartup ()
   throw (RuntimeException)
{
   CommandLine& cl (CommandLine::instantiate ());

   string domain = cl.getValue ("domain");

   http::wims20::ClientSide* wims20Request = NULL;

   if (cl.exists ("path")) {
      string path = cl.getValue ("path");
      wims20Request = new http::wims20::ClientSide (domain, path);
   }
   else
      wims20Request = new http::wims20::ClientSide (domain);

   wims20Request->setServiceID ("math");
   wims20Request->setGUID ("user@tid.es");
   wims20Request->addOtherLevel (cl.getValue ("op"));
   wims20Request->setParameter ("X", cl.getIntegerValue ("x"));
   wims20Request->setParameter ("Y", cl.getIntegerValue ("y"));

   http::Header* header = a_httpRequest.createHeader (http::Header::Type::Connection);
   header->setValue ("   Keep-Alive    ");

   if (cl.exists ("m")) {
      http::Method::Type::_v type = http::Method::Type::asEnumEx (cl.getValue ("m"));
      a_httpRequest.setMethod (type);
   }

   wims20Request->codeOn (a_httpRequest);

   static_cast <IHTTPClient&> (anna::comm::functions::getApp ()).getServer ()->send (a_httpRequest);
}

void MyCommunicator::eventReceiveMessage (ClientSocket& clientSocket, const Message& message)
   throw (RuntimeException)
{
   LOGMETHOD (TraceMethod tm ("MyCommunicator", "eventReceiveMessage", ANNA_FILE_LOCATION));

   if (clientSocket.support (http::Transport::className ()) == false)
      return;

   a_httpHandler.apply (clientSocket, message);
}

void MyHandler::evResponse (ClientSocket& clientSocket, const http::Response& response)
   throw (RuntimeException)
{
   cout << "HTTP StatusCode: " << response.getStatusCode () << endl;
   cout << "HTTP Text: " << response.getReasonPhrase () << endl;

   app::functions::component <Communicator> (ANNA_FILE_LOCATION)->requestStop ();
}

