// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <iostream>

#include <anna/core/core.hpp>
#include <anna/io/io.hpp>

#include <anna/app/Application.hpp>

class Test : public anna::app::Application {
public:
   Test ();
   
   void initialize () throw (anna::RuntimeException);
   
private:
   void run () throw (anna::RuntimeException);      
};

using namespace std;

int main (int argc, const char** argv)
{
   CommandLine& commandLine (CommandLine::instantiate ());
   Test test;

   try {
      commandLine.initialize (argv, argc);
      commandLine.verify ();

      Logger::setLevel (Logger::Debug); 
      Logger::initialize ("testfunctions", new TraceWriter ("file.trace", 2048000));
 
      test.start ();
   }
   catch (Exception& ex) {
      cout << ex.asString () << endl;
   }
   
   return 0;
}

Test::Test () : 
   app::Application ("testfunctions", "Comprobacion del sistema io", "1.0") 
{
   CommandLine& commandLine (CommandLine::instantiate ());
      
   commandLine.add ("file", CommandLine::Argument::Mandatory, "Nombre del fichero a procesar");
   commandLine.add ("mode", CommandLine::Argument::Mandatory, "Modo (binary|text)");
   commandLine.add ("size", CommandLine::Argument::Mandatory, "Tamano del buffer de entrada");
}

void Test::initialize () 
   throw (RuntimeException)
{
}

void Test::run () 
   throw (RuntimeException)
{
   CommandLine& commandLine (CommandLine::instantiate ());

   const char* mode = commandLine.getValue ("mode");
   int size = commandLine.getIntegerValue ("size");
   const char* file = commandLine.getValue ("file");

   if (anna_strcmp (mode, "binary") == 0) {
      BinaryReader reader (file, size);
      const DataBlock* dataBlock;

      while ((dataBlock = reader.fetch ()) != NULL) 
         cout << anna::functions::asString (*dataBlock, 24) << endl << endl;
   }
   else if (anna_strcmp (mode, "text") == 0) {
      TextReader reader (file, size);
      const char* line;
      while ((line = reader.fetch ()) != NULL) 
         cout << line << endl << endl;
   }
}

