// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_core_util_Encoder_hpp
#define anna_core_util_Encoder_hpp

#include <anna/core/RuntimeException.hpp>
#include <anna/core//DataBlock.hpp>
#include <anna/core/util/defines.hpp>
#include <anna/core/util/EncodedData.hpp>

#include <anna/core/util/Tokenizer.hpp>

namespace anna {

/**
   Interface generico para codificacion y decodificacion de bloques de memoria. Por ahora
   solo se aplica el Triple-DES pero es posible que un futuro se pueda selecionar algun otro
   algoritmo.
*/
class Encoder {
public:
  /**
     Constructor.
  */
  Encoder() {;}

  /**
     Codifica el texto recibido como parametro aplicando el algoritmo seleccionado.
     \param text Texto a codificar.
     \return Una representacion del texto codificado.
     \warning El texto codificado incluye el caracter cero, indicador de fin de cadena.
  */
  const EncodedData& encode(const char* text) throw(RuntimeException) {
    DataBlock aux(text, anna_strlen(text) + 1, false);
    return encode(aux);
  }

  /**
     Codifica el texto recibido como parametro aplicando el algoritmo seleccionado.
     \param text Texto a codificar.
     \return Una representacion del texto codificado.
     \warning El texto codificado incluye el caracter cero, indicador de fin de cadena.
  */
  const EncodedData& encode(const std::string& text) throw(RuntimeException) {
    return encode(text.c_str());
  }

  /**
     Codifica el bloque de memoria recibido como parametro aplicando el algoritmo seleccionado.
     \param data Bloque de memoria a codificar.
     \return Una representacion del bloque de memoria codificado.
  */
  const EncodedData& encode(const DataBlock& data) throw(RuntimeException);

  /**
     Deodifica la representacion de codificacion recibida como parametro.
     \param data Bloque de memoria a codificar.
     \return Una representacion del bloque de memoria codificado.

     \warning Si la representacion corresponde a la codificacion de una cadena con
     anna::DataBlock::getData obtendremos el contenido original de la cadena.
  */
  const DataBlock& decode(const EncodedData& data) throw(RuntimeException);

  /**
     Metodo que debe ser invocado al menos una vez.
  */
  static void initialize() throw();

private:
  EncodedData a_data;

  DataBlock& setDataBlock(const DataBlock& other) throw(RuntimeException);

};

}

#endif
