// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_diameter_comm_Transport_hpp
#define anna_diameter_comm_Transport_hpp


#include <anna/config/defines.hpp>
#include <anna/core/RuntimeException.hpp>
#include <anna/comm/Transport.hpp>
#include <anna/comm/TransportFactoryImpl.hpp>

namespace anna {
class DataBlock;
}



namespace anna {

namespace diameter {

namespace comm {


/**
   Clase generica para definir la capa de transporte del protocolo de comunicaciones diameter.

   Este protocolo esta orientado a intercambiar mensajes en una red interna, por lo que supone
   que no van a existir errores en los mensajes recibidos y que los mensajes son suficientemente cortos
   como para ocupar un unico paquete, por lo que no tiene ningun tipo de etiqueta especial que nos informe
   sobre el inicio y/o fin del mensaje.

   Los supuestos bajo los que se dise este protocolo facilitan el desarrollo de clases que ofrecen un
   gran rendimiento, pero imposibilitan el desarrollo del sistema de re-sincronizacin en caso de que alguno
   de los mensajes no cumpla los supuestos. Es decir, si nos llega un mensaje errneo nuestro proceso no sera
   capaz de volver a sincronizarse nunca mas, por lo que el sistema de proteccion integrado en anna.comm
   terminara cerrando la conexion.
*/
class Transport : public anna::comm::Transport {
public:
  /**
     Destructor
  */
  virtual ~Transport();

  /**
     Devuelve el gestor de capas de transporte asociado a esta clase.
     \return El gestor de capas de transporte asociado a esta clase.
  */
  static anna::comm::TransportFactory& getFactory() throw() { return st_factory; }

  /**
     Devuelve el literal que identifica de esta clase.
     \return el literal que identifica de esta clase.
  */
  static const char* className() throw() { return "diameter::comm::Transport"; }

private:
  static anna::comm::TransportFactoryImpl <anna::diameter::comm::Transport> st_factory;

  Transport();

  int calculeSize(const anna::DataBlock&) throw(anna::RuntimeException);
  const anna::comm::Message* decode(const anna::DataBlock&) throw(anna::RuntimeException);
  const anna::DataBlock& code(anna::comm::Message&) throw(anna::RuntimeException);

  friend class anna::Allocator <anna::diameter::comm::Transport>;
};

}
}
}

#endif

