// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_ldap_Request_hpp
#define anna_ldap_Request_hpp

#include <anna/ldap/defines.hpp>
#include <anna/ldap/ClassCode.hpp>

namespace anna {

namespace xml {
class Node;
}

namespace ldap {

class Session;

/**
   Prototipo de las peticiones que podemos realizar a un servidor LDAP.
*/
class Request {
public:
  /**
   * Define las acciones a realizar en caso de que el temporizador de la peticin expire.
   */
  struct OnExpiry { enum _v { Abandon, Ignore }; };

  /**
     Devuelve el tipo de la clase de esta peticion indicada en el contructor.
     \return El tipo de la clase de esta peticion indicada en el contructor.
  */
  ClassCode::_v getClassCode() const throw() { return a_classCode; }

  /**
   * Devuelve la accin a realizar en caso de que el temporizador asociado a esta peticin expire.
   * \return la accin a realizar en caso de que el temporizador asociado a esta peticin expire.
   */
  OnExpiry::_v getOnExpiry() const throw() { return a_onExpiry; }

  /**
   * Establece la accin a realizar en caso de que el temporizador asociado a esta peticin expire.
   * \param onExpiry Indica la accin a realizar en caso de que el temporizador asociado a esta peticin expire.
   *
   * \warning Establecer el valor OnExpiry::Ignore podra causar prdida de memoria y uso innecesario de recursos.
   */
  void setOnExpiry(const OnExpiry::_v onExpiry)  throw() { a_onExpiry = onExpiry; }

  /**
     Devuelve una cadena con la informacion mas relevante de esta instancia.
     \return Una cadena con la informacion mas relevante de esta instancia.
  */
  virtual std::string asString() const throw();

  /**
     Devuelve un documento XML con la informacion mas relevante de esta instancia.
     \param parent Nodo XML del que colgar la informacion referente a esta instancia.
     \return Un documento XML con la informacion mas relevante de esta instancia.
  */
  virtual xml::Node* asXML(xml::Node* parent) const throw();

  /**
     Inicializa los parametros de esta peticion.
  */
  virtual void clear() throw() = 0;

  /**
     Devuelve \em NULL si la \em std::string recibida como parametro esta vacia o el contenido de la
     misma en otro caso.
     \return \em NULL si la \em std::string recibida como parametro esta vacia o el contenido de la
     misma en otro caso.
     \warning Uso interno.
  */
  static const char* asCString(const std::string& value) throw() {
    return (value.empty() == true) ? NULL : value.c_str();
  }

  /**
     Devuelve \em "<null>" si la std::string recibida como parametro esta vacia o el contenido de la
     misma en otro caso.
     \return \em "<null>" si la std::string recibida como parametro esta vacia o el contenido de la
     misma en otro caso.
  */
  static const char* asText(const std::string& value) throw() {
    return (value.empty() == true) ? "<null>" : value.c_str();
  }

protected:
  /**
     Constructor.
     \param classCode Tipo de clase de esta peticion.
     \param onExpiry Indica la accin a realizar si el temporizador de esta transacin expira.
  */
  Request(const ClassCode::_v classCode, const OnExpiry::_v onExpiry) : a_classCode(classCode), a_onExpiry(onExpiry) {;}

private:
  ClassCode::_v a_classCode;
  OnExpiry::_v a_onExpiry;

  virtual IdMessage send(Session&) const throw() = 0;

  friend class Session;
};

}
}

#endif

