// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <iostream>
#include <algorithm>
#include <iomanip>
#include <fstream>

#include <time.h>
#include <unistd.h>
#include <signal.h>
#include <sys/wait.h>
#include <sys/types.h>

#include <anna/core/RuntimeException.hpp>
#include <anna/core/tracing/Logger.hpp>
#include <anna/core/functions.hpp>
#include <anna/core/tracing/TraceWriter.hpp>
#include <anna/core/tracing/TraceMethod.hpp>
#include <anna/core/internal/ModuleManager.hpp>
#include <anna/core/util/CommandLine.hpp>

#include <anna/app/Application.hpp>
#include <anna/app/Component.hpp>
#include <anna/app/internal/sccs.hpp>
#include <anna/app/functions.hpp>

#include <anna/xml/Node.hpp>
#include <anna/xml/Attribute.hpp>
#include <anna/xml/Compiler.hpp>

using namespace std;
using namespace anna;

// Web sites:
#define WEBSITE_PROJECT "http://redmine.teslayout.com/projects/anna-suite"
#define WEBSITE_GENERAL "http://www.teslayout.com"

app::Application* app::Application::st_application = NULL;

app::Application::Application(const char* shortName, const char* title, const char* version, const char* date, const char* time) :
  a_shortName(shortName),
  a_running(false),
  a_version(version),
  a_title(title),
  a_enableGPL(false) {
  sigset(SIGUSR1, handlerSignalUSR);
  sigset(SIGUSR2, handlerSignalUSR);
  sigset(SIGTERM, handlerSignalTerminate);
  sigignore(SIGINT);
  app::sccs::activate();
  a_version += functions::getArchitecture();
  a_pid = getpid();

  if(st_application == NULL)
    st_application = this;

  cout << a_shortName << " - " << a_title << ". Version " << a_version << endl;

  if(date || time) {
    cout << "Revision date: ";

    if(date) cout << date << " ";

    if(time) cout << time;

    cout << endl;
  }

  cout << "(c) Copyright 2001-2006 Eduardo Ramos Testillano & Francisco Antonio Ruiz Rayo" << endl;
  cout << "                        eduardo.ramos.testillano@gmail.com & cisco.tierra@gmail.com" << endl;
  cout << "                        (project site: " << WEBSITE_PROJECT << ")" << endl << endl;
  //cout << "                          - Project site: " << WEBSITE_PROJECT << endl;
  //cout << "                          - Main site:    " << WEBSITE_GENERAL << endl << endl;
}

app::Component* app::Application::find(const char* className)
throw() {
  Component* component;
  for(iterator ii = begin(), maxii = end(); ii != maxii; ii ++) {
    component = Application::component(ii);

    if(anna_strcmp(className, component->getClassName()) == 0)
      return component;
  }

  return NULL;
}

/**
 * (1) Si se ejecuta el metodo clone evita que los hijos que termina se queden como zombies.
 */
void app::Application::start()
throw(RuntimeException) {
  TraceMethod tm("app::Application", "start", ANNA_FILE_LOCATION);
  ModuleManager& moduleManager = ModuleManager::instantiate();

  if(a_running == true) {
    string msg("app::Application { Name; ");
    msg += a_title;
    msg += " | Already on execution";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  if(sigset(SIGCHLD, handlerChildTerminate) != 0)                   // (1)
    throw RuntimeException("Error installing terminate handler", ANNA_FILE_LOCATION);

  try {
    if(a_enableGPL == true) {
      cout << "This program is free software: you can redistribute it and/or modify" << endl;
      cout << "it under the terms of the GNU General Public License as published by" << endl;
      cout << "the Free Software Foundation, either version 3 of the License, or" << endl;
      cout << "(at your option) any later version." << endl << endl;
      cout << "This program is distributed in the hope that it will be useful," << endl;
      cout << "but WITHOUT ANY WARRANTY; without even the implied warranty of" << endl;
      cout << "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the" << endl;
      cout << "GNU General Public License for more details." << endl << endl;
      cout << "You should have received a copy of the GNU General Public License" << endl;
      cout << "along with this program. If not, see <http://www.gnu.org/licenses/>." << endl << endl;
    }

    cout << "ANNA Suite. Version " << functions::getVersion() << endl;
    cout << "Revision date: " << __DATE__ << " " << __TIME__ << endl;
    cout << "(c) Copyright 2001-2006 Eduardo Ramos Testillano & Francisco Antonio Ruiz Rayo" << endl;
    cout << "                        eduardo.ramos.testillano@gmail.com & cisco.tierra@gmail.com" << endl << endl;
    initialize();
    cout << "Activating application modules ....................." << endl;

    for(ModuleManager::const_iterator ii = moduleManager.begin(), maxii = moduleManager.end(); ii != maxii; ii ++) {
      cout << "\t Module " << ModuleManager::module(ii) << endl;
    }

    cout << endl;
    startComponents();
    a_running = true;
    run();
    a_running = false;
    stopComponents();
  } catch(RuntimeException& ex) {
    ex.trace();
    stopComponents();
    throw;
  }
}

void app::Application::startComponents()
throw(RuntimeException) {
  LOGMETHOD(TraceMethod tm("app::Application", "startComponents", ANNA_FILE_LOCATION));
  Component* component;

  for(iterator ii = begin(); ii != end(); ii ++) {
    component = Application::component(ii);
    LOGINFORMATION(
      std::string msg("Starting component | ");
      msg += component->asString();
      Logger::write(Logger::Information, msg, ANNA_FILE_LOCATION);
    );
    component->initialize();
  }
}

void app::Application::stopComponents()
throw(RuntimeException) {
  LOGMETHOD(TraceMethod tm("app::Application", "stopComponents", ANNA_FILE_LOCATION));
  Component* component;

  for(iterator ii = begin(); ii != end(); ii ++) {
    component = Application::component(ii);
    LOGINFORMATION(
      std::string msg("Stopping component | ");
      msg += component->asString();
      Logger::write(Logger::Information, msg, ANNA_FILE_LOCATION);
    );
    component->stop();
  }
}

/*
 * (1) Este el codigo que ejecuta la copia que acabamos de crear.
 *     (1.1) Limpia la lista de hijos que pudiera tener el padre.
 */
app::Application& app::Application::clone()
throw(RuntimeException) {
  if(anna::functions::supportMultithread() == true) {
    string msg("app::Application::clone | pid: ");
    msg += functions::asString((int) a_pid);
    msg += " | Application clone is only supported for ST applications";
    throw RuntimeException(msg, errno, ANNA_FILE_LOCATION);
  }

  const pid_t pid = ::fork();

  if(pid < 0) {
    string msg("app::Application::clone | pid: ");
    msg += functions::asString((int) a_pid);
    throw RuntimeException(msg, errno, ANNA_FILE_LOCATION);
  }

  Component* component;

  if(pid == 0) {                                                                   // (1)
    Logger::showPID(true);
    a_pid = getpid();
    a_pids.clear();                                                               // (1.1)

    try {
      for(iterator ii = begin(); ii != end(); ii ++) {
        component = Application::component(ii);
        LOGINFORMATION(
          std::string msg("Cloning component on child | ");
          msg += component->asString();
          Logger::write(Logger::Information, msg, ANNA_FILE_LOCATION);
        );
        component->do_cloneChild();
      }

      do_cloneChild();
    } catch(RuntimeException& ex) {
      ex.trace();
      exit(1);
    }
  } else {
    LOGWARNING(
      string msg("app::Application::clone | Parent pid: ");
      msg += functions::asString((int) a_pid);
      msg += " | Child pid: ";
      msg += functions::asString((int) pid);
      Logger::warning(msg, ANNA_FILE_LOCATION);
    );

    try {
      for(iterator ii = begin(); ii != end(); ii ++) {
        component = Application::component(ii);
        LOGINFORMATION(
          std::string msg("Cloning component on parent | ");
          msg += component->asString();
          Logger::write(Logger::Information, msg, ANNA_FILE_LOCATION);
        );
        component->do_cloneParent();
      }

      do_cloneParent();
      a_pids.push_back(pid);
    } catch(RuntimeException& ex) {
      string msg("app::Application::clone | Abort execution: ");
      msg  += msg += functions::asString((int) pid);
      Logger::error(msg, ANNA_FILE_LOCATION);
      kill(pid, SIGKILL);
      throw;
    }
  }

  return *this;
}

//---------------------------------------------------------------------------------------
// (1) Se invoca desde los constructores de los componentes => no se puede invocar a
// ningn m�odo virtual => obliga a que est� definidos antes de comenzar la
// ejecucin de la aplicacin => se inicilizar� en el Application::start ya que en
// ese momento los objetos est� completamente creados.
//---------------------------------------------------------------------------------------
void app::Application::attach(app::Component* component)
throw(RuntimeException) {
  if(component == NULL)
    throw RuntimeException("Cannot attach a NULL component", ANNA_FILE_LOCATION);

  iterator ii = std::find(begin(), end(), component);

  if(ii != end()) {
    LOGINFORMATION(
      //string msg((*ii)->asString());
      string msg((*ii)->getClassName());
      msg += " | Was previously attached !";
      Logger::write(Logger::Information, msg, ANNA_FILE_LOCATION);
    )
    return;
  }

  LOGDEBUG(
    string msg("app::Application::attach | ");
    msg += component->asString();
    Logger::debug(msg, ANNA_FILE_LOCATION);
  );
  a_components.push_back(component);

  if(a_running) {
    LOGINFORMATION(
      std::string msg("Starting component | ");
      msg += component->asString();
      Logger::write(Logger::Information, msg, ANNA_FILE_LOCATION);
    );
    component->initialize();
  }
}

void app::Application::detach(app::Component* component)
throw(RuntimeException) {
  LOGMETHOD(TraceMethod tm("app::Application", "detach(component)", ANNA_FILE_LOCATION));

  if(component == NULL)
    throw RuntimeException("Cannot detach a NULL component", ANNA_FILE_LOCATION);

  LOGDEBUG(
    string msg("app::Application::detach | ");
    msg += component->asString();
    Logger::debug(msg, ANNA_FILE_LOCATION);
  );
  iterator ii = std::find(begin(), end(), component);

  if(ii != end()) {
    LOGDEBUG(Logger::write(Logger::Debug, "Detached", (*ii)->asString(), ANNA_FILE_LOCATION));
    a_components.erase(ii);
  }
}

void app::Application::writeContext(const std::string& file)
throw(RuntimeException) {
  ofstream out;
  out.open(file.c_str());

  if(out.is_open() == false) {
    string msg("Error opening '");
    msg += file;
    msg += "'";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  LOGNOTICE(
    string msg("app::Application::writeContext | File: ");
    msg += file;
    Logger::notice(msg, ANNA_FILE_LOCATION);
  );
  xml::Node root("Application");
  out << xml::Compiler().apply(asXML(&root)) << endl;
  out.close();
}

xml::Node* app::Application::asXML(xml::Node* app) const
throw() {
  xml::Node* node(NULL);
  app->createAttribute("Name", getShortName());
  app->createAttribute("Version", getVersion());
  app->createAttribute("pid", getPid());
  // CommandLine:
  CommandLine::instantiate().asXML(app);
  node = app->createChild("app.Clones");

  for(const_pid_iterator ii = pid_begin(), maxii = pid_end(); ii != maxii; ii ++)
    node->createChild("Clone")->createAttribute("PID", pid(ii));

  ModuleManager& moduleManager = ModuleManager::instantiate();
  node = app->createChild("app.Modules");

  for(ModuleManager::const_iterator ii = moduleManager.begin(), maxii = moduleManager.end(); ii != maxii; ii ++)
    node->createChild("Module")->createAttribute("Id", ModuleManager::module(ii));

  const Component* cc;
  node = app->createChild("app.Components");

  for(const_iterator ii = begin(), maxii = end(); ii != maxii; ii ++) {
    Guard guard(cc = component(ii));
    cc->asXML(node);
  }

  return app;
}

void app::Application::signalUSR(int signal)
throw(RuntimeException) {
  if (signal == SIGUSR1) signalUSR1();
  else if (signal == SIGUSR2) signalUSR2();
}

void app::Application::signalUSR1()
throw(RuntimeException) {
  writeContext(anna::functions::asString("/var/tmp/anna.context.%05d", getPid()));
}

void app::Application::signalUSR2()
throw(RuntimeException) {
  Logger::notice("Captured signal SIGUSR2. Nothing implemented at the moment", ANNA_FILE_LOCATION);
}

void app::Application::sendSignalToChilds(const int signal)
throw() {
  for(pid_iterator ii = pid_begin(), maxii = pid_end(); ii != maxii; ii ++)
    kill(pid(ii), signal);
}

void app::Application::handlerSignalUSR(int signal)
throw() {
  sigignore(signal);

  try {
    Application& app = anna::app::functions::getApp();
    app.sendSignalToChilds(signal);
    app.signalUSR(signal);
  } catch(Exception& ex) {
    ex.trace();
  }

  sigset(signal, handlerSignalUSR);
}

void app::Application::handlerSignalTerminate(int)
throw() {
  sigignore(SIGTERM);

  try {
    LOGWARNING(
      Logger::write(Logger::Warning, "app::Application | Received SIGTERM", ANNA_FILE_LOCATION);
    );
    Application& app = anna::app::functions::getApp();
    app.sendSignalToChilds(SIGTERM);
    app.signalTerminate();
  } catch(Exception& ex) {
    ex.trace();
    exit(EXIT_SUCCESS);
  }

  sigset(SIGTERM, handlerSignalTerminate);
}

/**
 * Se recoge el estado de terminacion del hijo para que no se quede considerado como un zombie y se
 * elimina de la lista de procesos hijos.
 */
void app::Application::handlerChildTerminate(int sig)
throw() {
  if(sig == SIGCHLD) {
    int status;
    pid_t pid = wait(&status);

    if(st_application == NULL)
      return;

    pid_iterator maxii = st_application->pid_end();
    pid_iterator ii = ::find(st_application->pid_begin(), maxii, pid);

    if(ii != maxii)
      st_application->a_pids.erase(ii);
  }
}



