// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <algorithm>

#include <anna/core/tracing/Logger.hpp>
#include <anna/core/tracing/TraceMethod.hpp>

#include <anna/xml/Node.hpp>
#include <anna/xml/Attribute.hpp>

#include <anna/app/functions.hpp>

#include <anna/comm/Network.hpp>
#include <anna/comm/Host.hpp>
#include <anna/comm/Server.hpp>

#include <anna/comm/Communicator.hpp>
#include <anna/comm/IndexedDelivery.hpp>
#include <anna/comm/functions.hpp>

using namespace std;
using namespace anna;

void comm::IndexedDelivery::prepare(const int key)
throw(RuntimeException) {
  const int size = comm::Delivery::size();

  if(size == 0) {
    string msg(asString());
    msg += " | No resource has been attached";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  const int index = key % size;

  a_iikey = begin() + index;

  LOGDEBUG(
    string msg("anna::comm::IndexedDelivery::prepare | Key: ");
    msg += functions::asString(key);
    msg += " | ";
    msg += resource(a_iikey)->asString();
    Logger::debug(msg, ANNA_FILE_LOCATION);
  );
}

comm::Resource* comm::IndexedDelivery::do_apply()
throw(RuntimeException) {
  comm::Resource* result = NULL;

  if(a_iikey == comm::Delivery::end())
    return NULL;

  comm::Resource* w = result = comm::Delivery::resource(a_iikey);

  if(w->isAvailable() == false || w->isEnabled() == false) {
    LOGWARNING(
      string msg(asString());
      msg += " | ";
      msg += w->asString();
      msg += " | Unavailable";
      Logger::warning(msg, ANNA_FILE_LOCATION);
    );
    result = NULL;
  }

  if(a_mode == Mode::Flexible) {
    iterator end;
    iterator ii;
    end = a_iikey;

    if((ii = a_iikey + 1) == comm::Delivery::end())
      ii = comm::Delivery::begin();

    while(ii != end) {
      w = comm::Delivery::resource(ii);

      if(w->isAvailable() == true && w->isEnabled() == true) {
        result = w;
        break;
      }

      ii ++;

      if(ii == comm::Delivery::end())
        ii = comm::Delivery::begin();
    }
  }

  return result;
}

string comm::IndexedDelivery::asString() const
throw() {
  string result = className();
  result += " { ";
  result += comm::Delivery::asString();
  result += " | Mode: ";
  result += (a_mode == Mode::Strict) ? "Strict" : "Flexible";
  return result += " }";
}

xml::Node* comm::IndexedDelivery::asXML(xml::Node* parent) const
throw() {
  xml::Node* node = parent->createChild("comm.IndexedDelivery");
  node->createAttribute("Mode", (a_mode == Mode::Strict) ? "Strict" : "Flexible");
  comm::Service::asXML(node);
  return node;
}

