// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/core/mt/Guard.hpp>

#include <anna/comm/handler/Manager.hpp>

using namespace anna;
using namespace anna::comm;

template <> comm::Communicator* handler::Manager::BinderSocketAllocator::st_communicator = NULL;
template <> comm::Communicator* handler::Manager::ServerSocketAllocator::st_communicator = NULL;
template <> comm::Communicator* handler::Manager::LocalConnectionAllocator::st_communicator = NULL;
template <> comm::Communicator* handler::Manager::RemoteConnectionAllocator::st_communicator = NULL;
template <> comm::Communicator* handler::Manager::DatagramSocketAllocator::st_communicator = NULL;
template <> comm::Communicator* handler::Manager::ClientSocketAllocator::st_communicator = NULL;

void handler::Manager::initialize(Communicator* communicator)
throw() {
  BinderSocketAllocator::st_communicator = communicator;
  ServerSocketAllocator::st_communicator = communicator;
  LocalConnectionAllocator::st_communicator = communicator;
  RemoteConnectionAllocator::st_communicator = communicator;
  DatagramSocketAllocator::st_communicator = communicator;
  ClientSocketAllocator::st_communicator = communicator;
}

Handler* handler::Manager::createHandler(comm::BinderSocket* binderSocket)
throw(RuntimeException) {
  handler::BinderSocket* result = a_binderSockets.create();
  result->setup(binderSocket);
  return result;
}

Handler* handler::Manager::createHandler(comm::ServerSocket* serverSocket)
throw(RuntimeException) {
  handler::ServerSocket* result = a_serverSockets.create();
  result->setup(serverSocket);
  return result;
}

Handler* handler::Manager::createHandler(comm::LocalConnection* localConnection)
throw(RuntimeException) {
  handler::LocalConnection* result = a_localConnections.create();
  result->setup(localConnection);
  return result;
}

Handler* handler::Manager::createHandler(comm::RemoteConnection* remoteConnection)
throw(RuntimeException) {
  handler::RemoteConnection* result = a_remoteConnections.create();
  result->setup(remoteConnection);
  return result;
}

Handler* handler::Manager::createHandler(comm::DatagramSocket* datagramSocket)
throw(RuntimeException) {
  handler::DatagramSocket* result = a_datagramSockets.create();
  result->setup(datagramSocket);
  return result;
}

Handler* handler::Manager::createHandler(comm::ClientSocket* clientSocket)
throw(RuntimeException) {
  handler::ClientSocket* result = a_clientSockets.create();
  result->setup(clientSocket);
  return result;
}

void handler::Manager::releaseHandler(Handler* handler)
throw() {
  if(handler == NULL)
    return;

  switch(handler->getType()) {
  case Handler::Type::BinderSocket:
    a_binderSockets.release(static_cast <BinderSocket*>(handler));
    break;
  case Handler::Type::ServerSocket:
    a_serverSockets.release(static_cast <ServerSocket*>(handler));
    break;
  case Handler::Type::LocalConnection:
    a_localConnections.release(static_cast <LocalConnection*>(handler));
    break;
  case Handler::Type::RemoteConnection:
    a_remoteConnections.release(static_cast <RemoteConnection*>(handler));
    break;
  case Handler::Type::DatagramSocket:
    a_datagramSockets.release(static_cast <DatagramSocket*>(handler));
    break;
  case Handler::Type::ClientSocket:
    a_clientSockets.release(static_cast <ClientSocket*>(handler));
    break;
  default: break;
  }
}
