// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


// Local
#include <anna/diameter.comm/Message.hpp>
#include <anna/diameter.comm/ClientSession.hpp>
#include <anna/diameter.comm/ServerSession.hpp>
#include <anna/diameter/codec/functions.hpp>
#include <anna/diameter/defines.hpp>

#include <anna/xml/Node.hpp>
#include <anna/core/functions.hpp>
#include <anna/core/RuntimeException.hpp>
#include <anna/comm/ClientSocket.hpp>
#include <anna/core/tracing/Logger.hpp>

using namespace std;
using namespace anna::diameter;
using namespace anna::diameter::comm;



const char* Message::asText(const OnExpiry::_v rc)
throw() {
  static const char* text [] = { "Abandon", "Ignore", "Retransmit" };
  return text [rc];
}

string Message::asString() const
throw() {
  string result("diameter::comm::Message { ");
  result += "ClassCode: ";
  result += ClassCode::asText(a_classCode);
  result += " | OnExpiry: ";
  result += asText(a_onExpiry);
  result += " | Retries: ";
  result += anna::functions::asString(a_retries);
  return result += " }";
}

anna::xml::Node* Message::asXML(anna::xml::Node* parent) const
throw() {
  anna::xml::Node* result = parent->createChild("diameter.comm.Message");
  result->createAttribute("ClassCode", ClassCode::asText(a_classCode));
  result->createAttribute("OnExpiry", asText(a_onExpiry));
  result->createAttribute("Retries", anna::functions::asString(a_retries));
  //isRequest ...
  return result;
}

//void Message::clear ()
//   throw ()
//{
//   ::clear();
////   a_classCode = ClassCode::Undefined;
////   a_onExpiry = OnExpiry::Ignore;
//}


bool Message::fixRequestSequence(HopByHop hbh, EndToEnd ete) throw() {
  setRequestHopByHop(getHopByHop()); // original request hop-by-hop (backup)
  setRequestEndToEnd(getEndToEnd()); // original request end-to-end (backup)
  bool result = false;

  if(hbh != getRequestHopByHop()) {
    codec::functions::setHopByHop((anna::DataBlock&)getBody(), hbh);
    result = true;
  }

  if(a_endToEndSequenced) {
    if(ete != getRequestEndToEnd()) {
      codec::functions::setEndToEnd((anna::DataBlock&)getBody(), ete);
      result = true;
    }
  }

  LOGDEBUG(
    string msg("diameter::comm::fixRequestSequence { ");
    msg += "Hop by hop: ";
    msg += anna::functions::asString(getRequestHopByHop());
    msg += " (original) -> ";
    msg += anna::functions::asString(hbh);
    msg += " (session)";
    msg += a_endToEndSequenced ? " | End to end: " : " | End to end [end-to-end unchanged]: ";
    msg += anna::functions::asString(getRequestEndToEnd());
    msg += " (original) -> ";
    msg += anna::functions::asString(ete);
    msg += " (session)";
    anna::Logger::debug(msg, ANNA_FILE_LOCATION);
  );
  return result;
}



void Message::restoreSequencesAfterFix() throw() {
  LOGDEBUG(
    string msg("diameter::comm::restoreSequencesAfterFix { ");
    msg += "Hop by hop: ";
    msg += anna::functions::asString(getHopByHop());
    msg += " (session) -> ";
    msg += anna::functions::asString(getRequestHopByHop());
    msg += " (original)";
    msg += " | End to end: ";
    msg += anna::functions::asString(getEndToEnd());
    msg += " (session) -> ";
    msg += anna::functions::asString(getRequestEndToEnd());
    msg += " (original)";
    anna::Logger::debug(msg, ANNA_FILE_LOCATION);
  );
  diameter::codec::functions::setHopByHop((anna::DataBlock&)getBody(), a_requestHopByHop);
  diameter::codec::functions::setEndToEnd((anna::DataBlock&)getBody(), a_requestEndToEnd);
}


void Message::send(ClientSession& clientSession) const
throw(anna::RuntimeException) {
  try {
    clientSession.getServer()->send((Message *)this);
  } catch(anna::RuntimeException&) {
    throw;
  }
}

void Message::send(ServerSession& serverSession) const
throw(anna::RuntimeException) {
  try {
    serverSession.getClientSocket()->send((Message *)this);
  } catch(anna::RuntimeException&) {
    throw;
  }
}



anna::diameter::CommandId Message::getCommandId(bool &isRequest) const throw() {
  diameter::CommandId result = diameter::codec::functions::getCommandId(getBody());
  isRequest = result.second; // diameter::codec::functions::isRequest(result);
  return result;
}

anna::diameter::ApplicationId Message::getApplicationId() const throw() {
  diameter::ApplicationId result = diameter::codec::functions::getApplicationId(getBody());
  return result;
}

HopByHop Message::getHopByHop() const throw() {
  return (diameter::codec::functions::getHopByHop(getBody()));
}

EndToEnd Message::getEndToEnd() const throw() {
  return (diameter::codec::functions::getEndToEnd(getBody()));
}
