// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/core/tracing/Logger.hpp>
#include <anna/core/functions.hpp>

#include <anna/http/wims20/Abstract.hpp>

using namespace std;
using namespace anna;

http::wims20::Abstract::Abstract(const char* whatis, const std::string& domain, const std::string& path) :
  a_whatis(whatis),
  a_domain(domain) {
  a_path = createString(path);
  a_otherLevels = NULL;
  a_parameters = NULL;
}

http::wims20::Abstract::Abstract(const char* whatis, const std::string& domain) :
  a_whatis(whatis),
  a_domain(domain) {
  a_path = NULL;
  a_otherLevels = NULL;
  a_parameters = NULL;
}

/* virtual */
http::wims20::Abstract::~Abstract() {
  destroyString(a_path);
  clear();
  delete a_otherLevels;
  delete a_parameters;
}

/* Calcula la parte fija de la URI, se invoca desde el constructor. */
const string& http::wims20::Abstract::calculeFixedPart()
throw(RuntimeException) {
  if(a_serviceID.empty() == true) {
    string msg(asString());
    msg += " | ServiceID can not be NULL";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  if(a_guid.empty() == true) {
    string msg(asString());
    msg += " | GUID can not be NULL";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  if(a_fixedPart.empty() == false)
    return a_fixedPart;

  calculeShortFixedPart();
  appendWithSlash(a_fixedPart, a_serviceID);
  appendWithSlash(a_fixedPart, a_guid);
  return a_fixedPart;
}

const string& http::wims20::Abstract::calculeShortFixedPart()
throw(RuntimeException) {
  string::size_type pos = a_domain.find("http://");

  if(pos == string::npos) {
    a_fixedPart = "http://";
    a_fixedPart += a_domain;
  } else {
    if(pos > 0) {
      string msg("Domain '");
      msg += a_domain;
      msg += "' is not valid";
      throw RuntimeException(msg, ANNA_FILE_LOCATION);
    }

    a_fixedPart = a_domain;
  }

  if(a_path != NULL)
    appendWithSlash(a_fixedPart, *a_path);

  return a_fixedPart;
}

void http::wims20::Abstract::other_level_add(const std::string& otherLevel)
throw(RuntimeException) {
  if(a_otherLevels == NULL)
    a_otherLevels = new other_level_container;

  a_otherLevels->push_back(createString(otherLevel));
  LOGDEBUG(
    string msg("http::wims20::Abstract::other_level_add | Level: ");
    msg += otherLevel;
    Logger::debug(msg, ANNA_FILE_LOCATION);
  );
}

/*virtual*/
void http::wims20::Abstract::clearOtherLevels()
throw() {
  if(hasOtherLevels()) {
    for(other_level_iterator ii = other_level_begin(), maxii = other_level_end(); ii != maxii; ii ++)
      destroyString(otherLevel(ii));

    a_otherLevels->clear();
  }
}

void http::wims20::Abstract::parameter_set(const std::string& name, const std::string& value)
throw(RuntimeException) {
  bool exists = false;

  if(a_parameters == NULL)
    a_parameters = new parameter_container;

  for(parameter_iterator ii = parameter_begin(), maxii = parameter_end(); ii != maxii; ii ++) {
    if(*parameter_name(ii) == name) {
      *parameter_value(ii) = value;
      exists = true;
      break;
    }
  }

  if(exists == false) {
    string* paramName = createString(name);
    string* paramValue = createString(value);
    parameter_pkv item(paramName, paramValue);
    a_parameters->push_back(item);
  }

  LOGDEBUG(
    string msg("http::wims20::Abstract::parameter_set | Name: ");
    msg += name;
    msg += " | Value: ";
    msg += value;
    msg += " | New: ";
    msg += functions::asString(!exists);
    Logger::debug(msg, ANNA_FILE_LOCATION);
  );
}

void http::wims20::Abstract::clearParameters()
throw() {
  if(hasParameters()) {
    for(parameter_iterator ii = parameter_begin(), maxii = parameter_end(); ii != maxii; ii ++) {
      destroyString(parameter_name(ii));
      destroyString(parameter_value(ii));
    }

    a_parameters->clear();
  }
}

string http::wims20::Abstract::asString() const
throw() {
  string result("http::wims20::");
  result += a_whatis;
  result += " { Domain: ";
  result += a_domain;
  result += " | Path: ";
  result += (a_path == NULL) ? "<null>" : a_path->c_str();
  result += " | ServiceID: ";
  result += a_serviceID;
  result += " | GUID: ";
  result += a_guid;
  result += " | OtherLevels:";

  if(hasOtherLevels() == true) {
    for(const_other_level_iterator ii = other_level_begin(), maxii = other_level_end(); ii != maxii; ii ++) {
      result += ' ';
      result += otherLevel(ii);
    }
  } else
    result += " <null>";

  result += " | Parameters:";

  if(hasParameters() == true) {
    for(const_parameter_iterator ii = parameter_begin(), maxii = parameter_end(); ii != maxii; ii ++) {
      result += ' ';
      result += parameter_name(ii);
      result += '=';
      result += parameter_value(ii);
    }
  } else
    result += " <null>";

  return result += " }";
}

/*static*/
void http::wims20::Abstract::appendWithSlash(std::string& target, const std::string& other)
throw() {
  const char* str = target.c_str();

  if(str [anna_strlen(str) - 1] != '/')
    target += '/';

  target += other;
}
